TO_INJECT_FLAG= "__inject__"
INJECTED = "__injected__"
ANNOTATED = "__anno__"
ANNOTATIONS = "__annotations__"
POST_CONSTRUCT = "__post_construct__"
PRE_DESTROY = "__pre_destroy__"
SCOPE_PARAMETER = "__scopeton_scope__"

